﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file EV3Control_Python.py
 @brief EV3Control Python
 @date $Date$


"""
import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist

# Import ev3dev module
import ev3dev.ev3 as ev3

import math

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
ev3control_python_spec = ["implementation_id", "EV3Control_Python",
		 "type_name",         "EV3Control_Python",
		 "description",       "EV3Control Python",
		 "version",           "1.0.0",
		 "vendor",            "VenderName",
		 "category",          "Category",
		 "activity_type",     "STATIC",
		 "max_instance",      "1",
		 "language",          "Python",
		 "lang_type",         "SCRIPT",
		 ""]
# </rtc-template>

##
# @class EV3Control_Python
# @brief EV3Control Python
#
#
class EV3Control_Python(OpenRTM_aist.DataFlowComponentBase):

	##
	# @brief constructor
	# @param manager Maneger Object
	#
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

		self._d_vel = RTC.TimedVelocity2D(RTC.Time(0, 0),RTC.Velocity2D(0.0, 0.0, 0.0))
		"""
		"""
		self._velIn = OpenRTM_aist.InPort("vel", self._d_vel)
		
		Config=RTC.RangerConfig(1,2,3,4,5,6,7)
		Orientation3D=RTC.Orientation3D(0,0,0)
		Point3D=RTC.Point3D(0,0,0)
		Pose3D=RTC.Pose3D(Point3D,Orientation3D)
		Geometry3D = RTC.Geometry3D(Pose3D,RTC.Size3D(0,0,0))
		RangerGeometry=RTC.RangerGeometry(Geometry3D,[])
		
		self._d_ultrasonic = RTC.RangeData(RTC.Time(0, 0),[0.0],RangerGeometry,Config)
		"""
		"""
		self._ultrasonicOut = OpenRTM_aist.OutPort("ultrasonic", self._d_ultrasonic)
		self._d_color = RTC.TimedString(RTC.Time(0, 0),"1")
		"""
		"""
		self._colorOut = OpenRTM_aist.OutPort("color", self._d_color)
		self._d_touch = RTC.TimedBooleanSeq(RTC.Time(0, 0),[])
		"""
		"""
		self._touchOut = OpenRTM_aist.OutPort("touch", self._d_touch)





		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">

		# </rtc-template>



	##
	#
	# The initialize action (on CREATED->ALIVE transition)
	# formaer rtc_init_entry()
	#
	# @return RTC::ReturnCode_t
	#
	#
	def onInitialize(self):
		# Bind variables and configuration variable

		# Set InPort buffers
		self.addInPort("vel",self._velIn)

		# Set OutPort buffers
		self.addOutPort("ultrasonic",self._ultrasonicOut)
		self.addOutPort("color",self._colorOut)
		self.addOutPort("touch",self._touchOut)

		# Set service provider to Ports

		# Set service consumers to Ports

		# Set CORBA Service Ports
		#EV3
		self.motorA = ev3.LargeMotor('outA')
		self.motorB = ev3.LargeMotor('outB')
		self.d = 0.05925
		self.r = 0.028
		
		#Ultrasonic
		self.us = ev3.UltrasonicSensor('in3')
		#Touch
		self.touchleft = ev3.TouchSensor('in2')
		self.touchright = ev3.TouchSensor('in4')
		#ColorSensor
		self.color =ev3.ColorSensor('in1')
		print("Initialize")
		return RTC.RTC_OK

	###
	##
	## The finalize action (on ALIVE->END transition)
	## formaer rtc_exiting_entry()
	##
	## @return RTC::ReturnCode_t
	#
	##
	#def onFinalize(self):
	#
	#	return RTC.RTC_OK

	###
	##
	## The startup action when ExecutionContext startup
	## former rtc_starting_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onStartup(self, ec_id):
	#
	#	return RTC.RTC_OK

	###
	##
	## The shutdown action when ExecutionContext stop
	## former rtc_stopping_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onShutdown(self, ec_id):
	#
	#	return RTC.RTC_OK

	##
	#
	# The activated action (Active state entry action)
	# former rtc_active_entry()
	#
	# @param ec_id target ExecutionContext Id
	#
	# @return RTC::ReturnCode_t
	#
	#
	def onActivated(self, ec_id):
		print("Activate")
		return RTC.RTC_OK

	##
	#
	# The deactivated action (Active state exit action)
	# former rtc_active_exit()
	#
	# @param ec_id target ExecutionContext Id
	#
	# @return RTC::ReturnCode_t
	#
	#
	def onDeactivated(self, ec_id):
		print("Deactivate")
		self.motorA.stop()
		self.motorB.stop()
		return RTC.RTC_OK

	##
	#
	# The execution action that is invoked periodically
	# former rtc_active_do()
	#
	# @param ec_id target ExecutionContext Id
	#
	# @return RTC::ReturnCode_t
	#
	#
	def onExecute(self, ec_id):
		# Motor
		if self._velIn.isNew():
			self._d_vel=self._velIn.read()
			
			W_l = (self._d_vel.data.vx - self.d * self._d_vel.data.va)/self.r
			W_r = (self._d_vel.data.vx + self.d * self._d_vel.data.va)/self.r
			sp_l = int(W_l * (180 / math.pi)*(self.motorA.count_per_rot /360.0))
			sp_r = int(W_r * (180 / math.pi)*(self.motorB.count_per_rot /360.0))
			
			if sp_l >2000:
				sp_l=2000
			elif sp_l < -2000:
				sp_l=-2000
			if sp_r >2000:
				sp_r=2000
			elif sp_r < -2000:
				sp_r=-2000
			
			
			self.motorA.speed_sp=sp_l
			self.motorB.speed_sp=sp_r
			print("motorA sp",self.motorA.speed_sp)
			print("motorB sp",self.motorB.speed_sp)
			self.motorA.run_forever()
			self.motorB.run_forever()
		
		# Distance
		if self.us.connected:
			self._d_ultrasonic.ranges=[0.0]*1
			self._d_ultrasonic.ranges[0]=self.us.value()/100.0
			print("Distance = " ,self._d_ultrasonic.ranges[0])
			self._ultrasonicOut.write()
		# touch
		if self.touchleft.connected or self.touchright.connected:
			self._d_touch.data = [0]*2
			if self.touchleft.connected:
				self._d_touch.data[0] = self.touchleft.value()
			if self.touchright.connected:
				self._d_touch.data[1] = self.touchright.value()
			print("touch left = " ,self._d_touch.data[0] )
			print("touch right = " ,self._d_touch.data[1] )
			self._touchOut.write()
		# Color
		if self.color.connected:
			self._d_color.data =str(self.color.value())
			print("color = " ,self._d_color.data )
			self._colorOut.write()
		
		return RTC.RTC_OK

	###
	##
	## The aborting action when main logic error occurred.
	## former rtc_aborting_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onAborting(self, ec_id):
	#
	#	return RTC.RTC_OK

	###
	##
	## The error action in ERROR state
	## former rtc_error_do()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onError(self, ec_id):
	#
	#	return RTC.RTC_OK

	###
	##
	## The reset action that is invoked resetting
	## This is same but different the former rtc_init_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onReset(self, ec_id):
	#
	#	return RTC.RTC_OK

	###
	##
	## The state update action that is invoked after onExecute() action
	## no corresponding operation exists in OpenRTm-aist-0.2.0
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##

	##
	#def onStateUpdate(self, ec_id):
	#
	#	return RTC.RTC_OK

	###
	##
	## The action that is invoked when execution context's rate is changed
	## no corresponding operation exists in OpenRTm-aist-0.2.0
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onRateChanged(self, ec_id):
	#
	#	return RTC.RTC_OK




def EV3Control_PythonInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=ev3control_python_spec)
    manager.registerFactory(profile,
                            EV3Control_Python,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    EV3Control_PythonInit(manager)

    # Create a component
    comp = manager.createComponent("EV3Control_Python")

def main():
	mgr = OpenRTM_aist.Manager.init(sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

